package dashboard

import (
	"gorm.io/driver/mysql"
	"gorm.io/gorm"
	"gorm.io/gorm/logger"

	"shylinux.com/x/ice"
	"shylinux.com/x/icebergs/base/aaa"
	"shylinux.com/x/icebergs/base/ctx"
	"shylinux.com/x/icebergs/base/mdb"
	"shylinux.com/x/icebergs/base/web"
	kit "shylinux.com/x/toolkits"

	"shylinux.com/x/mysql-story/src/db"
	"shylinux.com/x/operation/src/dashboard/model"
)

type dashboard struct {
	Table
	order  string `data:"1"`
	fields string `data:"dashboard_type,host,port,username,database"`
	config string `name:"config host* port* username* password* database*" role:"leader"`
}

func (s dashboard) List(m *ice.Message, arg ...string) {
	if len(arg) == 0 {
		return
	}
	s.SelectDetail(m, model.UID, arg[0])
	if DashboardType(kit.Int(m.Append(model.DASHBOARD_TYPE))) == DashboardPlace {
		if s.IsLeader(m) {
			m.Action(s.Schema)
		}
		s.UserPlaceInit(m)
	} else if m.Append(model.USERNAME) == "" {
		m.SetAppend().EchoInfoButton("", s.Config)
	} else {
		if s.IsLeader(m) {
			m.PushAction(s.Config).Action(s.Config, s.Schema)
			s.UserPlaceInit(m)
		} else {
			m.Action()
		}
	}
}
func (s dashboard) Config(m *ice.Message, arg ...string) {
	s.Update(m, kit.Dict(arg), model.UID, m.Option(model.DASHBOARD_UID))
	s.UserPlaceInit(m)
}
func (s dashboard) Schema(m *ice.Message, arg ...string) {
	db := m.Configv(db.DB).(*gorm.DB)
	s.Show(m, db.Raw(`
		select TABLE_SCHEMA,TABLE_NAME,TABLE_ROWS,DATA_LENGTH,INDEX_LENGTH from information_schema.TABLES
		where TABLE_SCHEMA != 'mysql' AND TABLE_SCHEMA != 'information_schema' AND TABLE_SCHEMA != 'performance_schema'
		order by DATA_LENGTH desc
`))
	m.Display("/plugin/table.js")
}

func init() { ice.TeamCtxCmd(dashboard{}) }

func (s dashboard) Open(msg *ice.Message, cb func(*gorm.DB)) {
	dsn := kit.Format("%s:%s@tcp(%s:%s)/%s?charset=utf8mb4&parseTime=True",
		msg.Append(model.USERNAME), msg.Append(model.PASSWORD), msg.Append(model.HOST), msg.Append(model.PORT), msg.Append(model.DATABASE))
	if db, err := gorm.Open(mysql.Open(dsn), &gorm.Config{Logger: logger.Default.LogMode(logger.Info)}); !msg.Warn(err) {
		cb(db)
	}
}
func (s dashboard) Scan(m *ice.Message, dashboard_uid string, cb func(*ice.Message, ice.Maps)) {
	msg := m.Cmd(s, s.Select, model.UID, dashboard_uid)
	switch DashboardType(kit.Int(msg.Append(model.TYPE))) {
	case DashboardPlace:
		m.Table(func(value ice.Maps) {
			msg := s.SpaceCmd(m, value[web.SPACE], value[ctx.INDEX], mdb.SELECT, kit.Simple(kit.UnMarshal(value[model.QUERY])), kit.Dict(
				mdb.SELECT, kit.Select("count(*)", value[mdb.FIELD]), ice.MSG_USERROLE, aaa.TECH,
			))
			cb(msg, value)
		})
	case DashboardMySQL:
		s.Open(m, func(db *gorm.DB) {
			m.Table(func(value ice.Maps) {
				db = db.Table(s.TableName(kit.Join(kit.Slice(kit.Split(value[ctx.INDEX], "."), -2), ".")))
				kit.If(value[model.QUERY], func(p string) { s.Where(m, db, kit.Simple(kit.UnMarshal(p))...) })
				msg := s.Rows(m.Spawn(), db.Select(kit.Select("count(*)", value[mdb.FIELD])))
				cb(msg, value)
			})
		})
	case DashboardRedis:
	}
}

type DashboardType int

const (
	DashboardPlace DashboardType = iota
	DashboardMySQL
	DashboardRedis
)

var DashboardTypeList = map[DashboardType]string{
	DashboardPlace: "place",
	DashboardMySQL: "mysql",
	DashboardRedis: "redis",
}

func (s DashboardType) String() string { return DashboardTypeList[s] }
